<?php
/***************************************************************************
 *   Copyright (C) 2005-2020 by Ferenc Veres							   *
 *   lion@netngine.hu													   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/
include_once("lib/TranslationProject.php");

/**
 * Imports a Xaraya translation from locale files into the database.
 *
 * Importing Xaraya works the following way:
 *
 * Directories will be created as projects without documents, XML files will be
 * stored as projects with document. XML files are parsed for English and
 * Translated strings.
 *
 * If directory or file project already exist, the timestamp and in the later
 * case the original and skeleton documents are updated. (Name, parentid and
 * localeid identifies the existing projects.) If the timestamp of a project
 * is smaller than the root project's, that project was not found while the
 * last import, thus won't be exported and considered to be dead project file.
 *
 * If the same project contains the same English original string already, the
 * string entry will be updated. If there is no such string yet in this project,
 * a new string will be created and indexed.
 */

class TranslationImportXaraya implements TranslationImport
{
	protected $ProjectName = "Xaraya";
	protected $XmlNameSpace = "http://xaraya.com/2002/ns/translations";

	private $LocaleID;

	/**
	* Class to import Xaraya locale files.
	*/
	
	function __construct()
	{
	}
	
	/**
	* Imports one translation XML file into the databse.
	*
	* @param $fullName The path and name of the file to import.
	* @param $thisProject The project in which this one will be stored.
	*/
	
	private function ImportFile($fullName, TranslationProject $thisProject)
	{
		// Load the XML file
		$langXml = new DOMDocument();
		if(!$langXml->load($fullName))
		{
			throw new Exception("Can't load language XML file.\n");
		}

		// Store the document in the project
		$document = $langXml->saveXML();
		$thisProject->SetDocument($document);
		
		// XPath to all translation entries
		$xpath = new DOMXPath($langXml);
		$xpath->registerNamespace("m", $this->XmlNameSpace);
		$entryNodes = $xpath->query("//m:translations/m:entry");

		$projID = $thisProject->GetID();
		$locID = $thisProject->GetLocaleID();

		$rowCount = 0;
		
		// Loop on each string
		foreach ($entryNodes as $entry)
		{
			$origNodes = $xpath->query("m:string", $entry);
			$transNodes = $xpath->query("m:translation", $entry);

			if($origNodes->item(0)->nodeValue == "")
			{
				echo "Empty Original node in the current file.\n";
			}
			else
			{
				// Stores translation and create index if not yet done
				$str = new TranslationString(
					0,
					$projID,
					$origNodes->item(0)->nodeValue, 
					$locID, 
					$transNodes->item(0)->nodeValue,
					0);
				$str->SetRowOrder($rowCount++);
				$id = $str->Save();
				
				$origNodes->item(0)->nodeValue = "###${id}###";
				$transNodes->item(0)->nodeValue = "###${id}t###";

			}
		}

		// Store the finished skeleton in the project
		$skeleton = $langXml->saveXML();
		$thisProject->SetSkeleton($skeleton);
		$thisProject->Save();
	}

	/**
	* Recursively process a complete dir and call ImportFile() for each XML file.
	*
	* The root project must be created before this function is called!
	* Pass the root project to this function!
	*
	* @param $dirIterator A directory iterator initialized with the start dir, usually your Xaraya locale's dir.
	* @param $parentProject The parent project, in which this new subprojects will be created.
	*/

	function ProcessDir(RecursiveDirectoryIterator $dirIterator, TranslationProject $parentProject)
	{
		$root = ($parentProject->GetRootID() == 0 ? $parentProject->GetID() : $parentProject->GetRootID());

		foreach($dirIterator as $file)
		{
			$fname = preg_replace('/.*\//', '', $file->getFilename()); // FIXME: required due to a bug in php5.1.2
			if($fname == "." || $fname == "..") 
			{
				continue;
			}

			//if($file->isDir() || ($file->isFile() && preg_match("/\.xml$/", $fname)))
			//{
				// Create new subproject for ourselves or get existing one
				$newProject = new TranslationProject();
	
				if(!$newProject->Load($parentProject->GetID(), $fname, $parentProject->GetLocaleID(), false))
				{
					$newProject->CreateProject($root, $parentProject->GetID(), $fname, $file->isDir(), $parentProject->GetLocaleID());
				}
				
				// Save new or update existing one's timestamp
				$newProject->Save();
	
				// Process file
				if($file->isFile())
				{
					if(preg_match("/\.xml$/", $fname))
					{
						// Process locale files.
						echo "process_file(".$file->getPathname().")\n";
						$this->ImportFile($file->getPathname(), $newProject);
					}
					else
					{
						// Store nonXML files (e.g. index.html)
						echo "store file(".$file->getPathname().")\n";
						$newProject->SetDocument(file_get_contents($file->getPathname()));
						$newProject->Save();
					}
				}
	
				// Go for subdirs
				if($file->isDir()) 
				{
					$this->ProcessDir($dirIterator->getChildren(), $newProject);
				}
			//}
		}
	}

	/**
	* Start importing the the selected directory into the selected locale.
	*
	* Uses $CONFIG['import_dir'] to find the files.
	* @param $localeName The locale name which translation locale files should be imported.
	*/
	
	function Import($localeName)
	{
		$localeID = TranslationProject::$LocaleIDCache[$localeName];
		if(empty($localeID))
		{
			throw new Exception("ExportXaraya called with unknown locale name: $localeName");
		}
		
		// Clear cached project lists
		ProjectIterator::ClearCache($localeID);
		
		
		$dir = GetConfigVar('import_path')."/".$localeName."/xml/";
		if(!is_dir($dir))
		{
			throw new Exception("ExportXaraya using nonexisting directory: $dir");
		}

		$this->LocaleID = $localeID;

		// Load or create root project
		$rootProject = new TranslationProject();

		// Do we have a root for our project?
		if(!$rootProject->LoadRoot($this->ProjectName, $this->LocaleID))
		{
			// No root yet, create it now.
			$rootProject->CreateRootProject($this->ProjectName, $this->LocaleID);
		}
		
		// Save new project or update old one's timestamp
		if(!$rootProject->Save())
		{
			throw new Exception("Cannot create root project for Xaraya.");
		}

		// Start the recursive madness
		$dirIterator = new RecursiveDirectoryIterator($dir);
		$this->ProcessDir($dirIterator, $rootProject);
	}
}

?>
