<?php
/***************************************************************************
 *   Copyright (C) 2005-2020 by Ferenc Veres							   *
 *   lion@netngine.hu													   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/
include_once('TranslationProject.php');

/**
* Class to list and iterate TranslationProject classes as they are stored in the database.
*
* ProjectIterator can be initialized with a parent project id, and then it can
* be used to loop on child projects, either recursve or direct childs.
* ***** A 'PROJECT' IS ONE FILE IN XARAYA TERMS *****
*/

class ProjectIterator implements Iterator, RecursiveIterator
{
	/** Work with strings from this locale. */
	private $LocaleID;
	/** Start listing childs of this project. */
	private $ParentID;

	/** ID of the current project in the Iterator. */
	private $CurrentProject;
	
	/**  True value loads partial project details, without Skeleton and Document */
	private $Brief;

	/**  Cached list of all available projects e.g. ProjectList[2][44] = a TranslationProject */
	public static $ProjectList;	
	/** Cached array to hold project tree data */
	public static $ProjectChildren;	

	/**
	* Create a new TranslationProject iterator for looping on child projects.
	*
	* @param $parentID int The project ID in which we will loop child projects.
	* @param $localeID int The locale to work with.
	* @param $brief bool True value will load partial project details, without Skeleton and Document. 
	*/
	public function __construct($parentID, $localeID, $brief = false)
	{
		global $CONFIG;
		
		$this->Brief = $brief;
		
		$DB =& GetDbConn();
		$Tables =& GetTables();

		$this->LocaleID = $localeID;
		$this->ParentID = $parentID;
		
		// Create cache file names
		if($CONFIG['disk_cache'])
		{
			// These names are duplicated in ClearCache!!!
			$cacheFile = $CONFIG['disk_cache']."/ProjectList_".$this->LocaleID.".cache";
			$cacheFile2 = $CONFIG['disk_cache']."/ProjectChildren_".$this->LocaleID.".cache";
			if($this->Brief)
			{
				$cacheFile .= ".brief";
				$cacheFile2 .= ".brief";
			}
		}
		
		// Cache all projects for the current language, unless done already
		if(empty(self::$ProjectList[$this->LocaleID]))
		{
			// Load from file cache
			if($CONFIG['disk_cache'] && file_exists($cacheFile) && file_exists($cacheFile2))
			{
				self::$ProjectList[$this->LocaleID] = unserialize(file_get_contents($cacheFile));
				self::$ProjectChildren[$this->LocaleID] = unserialize(file_get_contents($cacheFile2));
			}

			// No file cache, reload from DB.
			if(empty(self::$ProjectList[$this->LocaleID]))
			{
				if($this->Brief)
				{
					$sql = "SELECT ID, RootID, ParentID, Name, IsFolder, LocaleID,".
					"TIMESTAMP(LastSeen) as LastSeen ".
					"FROM ".$Tables["Projects"]." WHERE LocaleID = ? ORDER BY Name";
				}
				else
				{
					$sql = "SELECT ID, RootID, ParentID, Name, IsFolder, LocaleID,".
					"Skeleton, OrigDoc,  TIMESTAMP(LastSeen) as LastSeen ".
					"FROM ".$Tables["Projects"]." WHERE LocaleID = ? ORDER BY Name";
				}
				$rs = $DB->Execute($sql, $this->LocaleID);
				$haveMore = true;
				
				while($haveMore)
				{
					$proj = new TranslationProject();
					
					if(($haveMore = $proj->LoadFromRecordSet($rs, $this->Brief)))
					{
						self::$ProjectList[$this->LocaleID][$proj->GetID()] = $proj;
						self::$ProjectChildren[$this->LocaleID][$proj->GetParentID()][] = $proj->GetID();
					}
				}
				$rs->Close();

				// Store in file cache
				if($CONFIG['disk_cache'])
				{
					file_put_contents($cacheFile, serialize(self::$ProjectList[$this->LocaleID]));
					file_put_contents($cacheFile2, serialize(self::$ProjectChildren[$this->LocaleID]));
				}
			}
		}
	
		if(empty(self::$ProjectList[$this->LocaleID]))
		{
			throw new Exception("There are no projects for Language ID: ".$this->LocaleID.". Please import before using.");
		}

		// Select the first child by default for ->valid()-ity check.
		$this->rewind();
	}

	/**
	 * Clear cached ProjectList and ProjectChildren (both memory and disk).
	 *
	 */
	public static function ClearCache($localeID)
	{
		global $CONFIG;
		
		ProjectIterator::$ProjectList = null;
		ProjectIterator::$ProjectChildren = null;

		if($CONFIG['disk_cache'])
		{
			$cacheFile = $CONFIG['disk_cache']."/ProjectList_".$localeID.".cache";
			$cacheFile2 = $CONFIG['disk_cache']."/ProjectChildren_".$localeID.".cache";
			
			if(file_exists($cacheFile))
			{
				unlink($cacheFile);
			}
			if(file_exists($cacheFile2))
			{
				unlink($cacheFile2);
			}
			if(file_exists($cacheFile.".brief"))
			{
				unlink($cacheFile.".brief");
			}
			if(file_exists($cacheFile2.".brief"))
			{
				unlink($cacheFile2.".brief");
			}
		}
	}
	/* Iterator interface */

	/**
	* Iterator interface: rewind function.
	*/
	public function rewind()
	{
		// Take first child
		if(!empty(self::$ProjectChildren[$this->LocaleID][$this->ParentID][0]))
		{
			$this->CurrentID = self::$ProjectChildren[$this->LocaleID][$this->ParentID][0];
		}
		else
		{
			$this->CurrentID = 0;
		}
	}
	/**
	* Iterator interface: hasMore function.
	*/
	public function hasMore()
	{
		// Get an array of our sibling' IDs
		$siblings = self::$ProjectChildren[$this->LocaleID][
			self::$ProjectList[$this->LocaleID][$this->CurrentID]->GetParentID()
			];

		// Loop on each sibling
		$takeNext = false;
		$foundMore = false;
		foreach($siblings as $pid)
		{
			// Found the next sibling?
			if($takeNext)
			{
				$foundMore = true;
				break;
			}

			// Is this the current item? If yes, take next sibling in the next round.
			if($pid == $this->CurrentID)
			{
				$takeNext = true;
			}
		}
		return $foundMore;
	}
	/**
	* Iterator interface: key function.
	*/
	public function key()
	{
		return $this->CurrentID;
	}
	/**
	* Iterator interface: current function.
	*/
	public function current()
	{
		return self::$ProjectList[$this->LocaleID][$this->CurrentID];
	}
	/**
	* Iterator interface: next function.
	*/
	public function next()
	{
		// Get an array of our sibling' IDs
		$siblings = self::$ProjectChildren[$this->LocaleID][
			self::$ProjectList[$this->LocaleID][$this->CurrentID]->GetParentID()
			];

		// Note the last ID and set like if we had no more (will change if we have)
		$lastID = $this->CurrentID;
		$this->CurrentID = 0;
		
		// Loop on each sibling
		$takeNext = false;
		foreach($siblings as $pid)
		{
			// Found the next sibling?
			if($takeNext)
			{
				$this->CurrentID = $pid;
				break;
			}
			
			// Is this the current item? If yes, take next sibling in the next round.
			if($pid == $lastID)
			{
				$takeNext = true;
			}
		}
	}
	/**
	* Iterator interface: valid function.
	*/
	public function valid()
	{
		return $this->CurrentID != 0;
	}

	/* Recursive Iterator Interface */

	/**
	* Recursive Iterator: hasChildren function
	*/
	public function hasChildren()
	{
		// Check if the children array has items with the current id as parent
		return !empty(self::$ProjectChildren[$this->LocaleID][$this->CurrentID]);
	}

	/**
	* Recursive Iterator: getChildren function
	*/
	public function getChildren()
	{
		// Make a new iterator below ourselves
		return new ProjectIterator($this->CurrentID, $this->LocaleID, $this->Brief);
	}
}
?>
