<?php
/***************************************************************************
 *   Copyright (C) 2005-2020 by Ferenc Veres							   *
 *   lion@netngine.hu													   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

/**
 * This class can handle projects. A project is either a file which contains
 * some translatable strings, or an empty project which does not have actual
 * strings, just subprojects only.
 *
 * A project(file) has an original document, a skeleton which contains ###33###
 * type references to TranslationString objects. The translated document can be
 * created from the Project's Skeleton and the Strings referenced by the ## IDs.
 * ***** A 'PROJECT' IS ONE FILE IN XARAYA TERMS *****
 */
 
class TranslationProject
{
	private $ProjectID;	/**< Project's ID if loaded */
	private $ParentID;	/**< Project's parent project id */
	private $RootID;    /**< Root element containing this project - redundant database data but fast */
	private $Name;		/**< Project name */
	private $IsFolder;  /**< True if the project it no a translatable document, just a folder or directory */
	private $LocaleID;	/**< Project locale ID */
	private $Skeleton;	/**< Project's original file with placeholders for the translatable strings */
	private $Document;	/**< The original untranslated, unchanged file of this project */
	private $LastSeen;	/**< The date and time (UNIX timestamp) when this project was last updated. */
	private $Brief;		/**< The project did not load Document and Skeleton for performance. Save not possible */
	
	public static $LocaleIDCache; /**< Static hash table caching the locale names(key) and IDs(value) */
	
	/**
	* Initialize a new project handler.
	*
	* @param $loadByID Integer, a project ID to be loaded. If not specified, empty project handler is made.
	* @param $brief If true, the faster project won't handle Document and Skeleton and Save won't work.
	*/

	public function __construct($loadByID = 0, $brief = false)
	{
		if($loadByID != 0)
		{
			$this->LoadByID($loadByID, $brief);
		}
	}

	/**
	* Initialize the static locale ID cache, by loading locales from the database.
	*
	* You can access the locale IDs in TraslationProject::$LocaleIDCache[localename] = id format.
	*/
	
	public static function InitLocaleIDCache()
	{
		$DB =& GetDbConn();
		$Tables =& GetTables();
		
		$rs = $DB->Execute("SELECT ID, Name FROM ".$Tables["Locales"]." ORDER BY Name");
		while($row = $rs->FetchRow()) 
		{
			TranslationProject::$LocaleIDCache[$row["Name"]] = $row["ID"];
		}
	}

	/**
	* Load array of all root projects of the given locale.
	*
	* @returns Array Hastable with Project ID as index and TranslationProject objects as data.
	*/

	public static function GetRoots($localeID) 
	{
		$DB =& GetDbConn();
		$Tables =& GetTables();

		$roots = array();
		$rs = $DB->Execute("SELECT ID FROM ".$Tables["Projects"]." WHERE ParentID = 0 AND LocaleID = ? ORDER BY Name", array($localeID));
		while($row = $rs->FetchRow()) 
		{
			$proj = new TranslationProject();
			$proj->LoadByID($row["ID"], false);
			$roots[$row["ID"]] = $proj;
		}
		return $roots;
	}

	/**
	* Reinitalize - clear all internal data of - this Project.
	*/
	
	private function Clear()
	{
	}

	/**
	* Check if there is another Project in the database with the same name,
	* in the same tree branch and the in the same locale (as the current Project properties).
	*
	* @returns Bool true if names collide, false if name is free
	*/
	
	private function CheckNameCollision()
	{
		$DB =& GetDbConn();
		$Tables =& GetTables();

		$old = $DB->GetOne("SELECT ID FROM ".$Tables["Projects"]." WHERE ParentID = ? AND Name = ? AND LocaleID = ?", array($this->ParentID, $this->Name, $this->LocaleID));
		if($old)
		{
			return true;
		}
		else
		{
			return false;
		}

	}

	/**
	* Create a new Project into the current object instance, which has 0 as parent ID.
	*
	* Root projects cannot have actual strings content, just a placeholder for
	* all its subprojects
	*
	* @param $name Name of the project to be created.
	* @param $localeID Locale ID of this project.
	* @returns The ID of the newly created project, false if a root project wiht $name already exists.
	*/
	
	public function CreateRootProject($name, $localeID)
	{
		return $this->CreateProject(0, 0, $name, true, $localeID);
	}

	/**
	* Initialize this object as a new (sub)project in a root or any other project.
	*
	* These projects may or may not store the real textural content for the project's translation.
	* If you want to store the project in the database, you must call ->Save() too. Until saving,
	* the project won't have a Project ID!
	* 
	* @param $parentID In which project to create this project as child.
	* @param $name Name of the project to be created.
	* @param $isFolder True if the project is a folder (no document, container for more projects)
	* @param $localeID Locale ID of this project.
	* @param $document The original content of the document to be translated.
	* @param $skeleton The document which contains IDs in place of each translation string from origdoc.
	* @exception 'Exception' thrown if a project with same name exists in the same location.
	*/
	
	public function CreateProject($rootID, $parentID, $name, $isFolder, $localeID, $document = '', $skeleton = '')
	{
		$this->RootID = $rootID;
		$this->ParentID = $parentID;
		$this->Name = $name;
		$this->IsFolder = $isFolder;
		$this->LocaleID = $localeID;
		$this->Document = $document;
		$this->Skeleton = $skeleton;
		$this->LastSeen = time();
		$this->Brief = false; // Newly created project is never brief, as we most likely want to save it
		
		// Project with this name already exists in this Parent?
		if($this->CheckNameCollision())
		{
			$this->Clear();
			throw new Exception("Project with this name already exists, same location.");
		}
	}

	/**
	* Save the current Project in the database (insert or update).
	*
	* The project may have empty skeleton and original document, as a placeholder project.
	* But name, parentid and LocaleID are mandatory to be set.
	*
	* @returns The ID of the created project.
	*/
	
	public function Save()
	{
		$DB =& GetDbConn();
		$Tables =& GetTables();
		
		if($this->IsFolder && ($this->Document != '' || $this->Skeleton != ''))
		{
			throw new Exception("Folder project must not contain a document or skeleton.");
		}
		
		if($this->Brief)
		{
			throw new Exception("Brief=True projects cannot be saved or updated. They are read-only.");
		}
		
		// Is this an already saved project?
		if(isset($this->ProjectID))
		{
			// Already saved (or a loaded) project, UPDATE in database (except RootID)
			
			$ok = $DB->Execute("UPDATE ".$Tables["Projects"]." SET ParentID = ?, Name = ?, IsFolder = ?, LocaleID = ?, OrigDoc = ?, Skeleton = ?, LastSeen = now() WHERE ID = ?", array($this->ParentID, $this->Name, ($this->IsFolder ? 1 : 0), $this->LocaleID, $this->Document, $this->Skeleton, $this->ProjectID));
		}
		else
		{
			// New project, INSERT into database
			
			// Project with this name already exists in this Parent?
			if($this->CheckNameCollision())
			{
				throw new Exception("Project with this name already exists, same location.");
			}
			
			$root = ($this->RootID > 0 ? $this->RootID : null);
			$ok = $DB->Execute("INSERT INTO ".$Tables["Projects"]." (RootID, ParentID, Name, IsFolder, LocaleID, OrigDoc, Skeleton, LastSeen) VALUES (?,?,?,?,?,?,?, now())", array($root, $this->ParentID, $this->Name, ($this->IsFolder ? 1 : 0), $this->LocaleID, $this->Document, $this->Skeleton));

			// Get new ID
			if(!$DB->Insert_ID())
			{
				throw new Exception("DB Error: Insert_ID not supported.");
			}
			$this->ProjectID = $DB->Insert_ID();
		}

		return $this->ProjectID;
	}

	/**
	* Load a root Project with the given name (which has parent ID 0).
	*
	* @param $name Which root project to look for.
	* @param $localeID Locale ID of the requested project.
	* @return True on success, false otherwise.
	*/
	
	public function LoadRoot($name, $localeID)
	{
		return $this->Load(0, $name, $localeID, false);
	}

	/**
	* Load a Project by name.
	*
	* @param $parentID In which parent project.
	* @param $name Which project to look for.
	* @param $localeID Which locale version to find.
	* @return True on success, false otherwise.
	*/
	
	public function Load($parentID, $name, $localeID, $brief)
	{
		$DB =& GetDbConn();
		$Tables =& GetTables();

		$this->Brief = $brief;
		if($this->Brief)
		{
			$sql = "SELECT ID, RootID, ParentID, Name, IsFolder, LocaleID,".
				"TIMESTAMP(LastSeen) as LastSeen ".
				"FROM ".$Tables["Projects"].
				" WHERE ParentID = ? AND Name = ? AND LocaleID = ?"; 
		}
		else
		{
			$sql = "SELECT ID, RootID, ParentID, Name, IsFolder, LocaleID,".
				"Skeleton, OrigDoc, TIMESTAMP(LastSeen) as LastSeen ".
				"FROM ".$Tables["Projects"].
				" WHERE ParentID = ? AND Name = ? AND LocaleID = ?";
		}

		$rs = $DB->Execute($sql, array($parentID, $name, $localeID));
		
		$retVal = $this->LoadRecordSetEntry($rs);
		$rs->Close();

		return $retVal;
	}


	/**
	* Load the next Project from an existing ADODB recordset.
	*
	* The query running in the recordset must return the same columns as the
	* query in this class' ->Load() method. Useful for loading a lot of project at once.
	*
	* @param $recordSet Existing ADODB recordset running a project list query.
	* @return True on success, false otherwise.
	*/
	
	public function LoadFromRecordSet($recordSet, $brief)
	{
		$this->Brief = $brief;
		return $this->LoadRecordSetEntry($recordSet);
	}

	/**
	* Load a project by ID.
	*
	* @param $projectID Which project to load.
	* @return True on success, false otherwise.
	*/
	
	public function LoadByID($projectID, $brief)
	{
		$DB =& GetDbConn();
		$Tables =& GetTables();

		$this->Brief = $brief;
		if($this->Brief)
		{
			$sql ="SELECT ID, RootID, ParentID, Name, IsFolder, LocaleID,".
				"TIMESTAMP(LastSeen) as LastSeen ".
				"FROM ".$Tables["Projects"]." WHERE ID = ?"; 
		}
		else
		{
			$sql ="SELECT ID, RootID, ParentID, Name, IsFolder, LocaleID,".
				"Skeleton, OrigDoc, TIMESTAMP(LastSeen) as LastSeen ".
				"FROM ".$Tables["Projects"]." WHERE ID = ?";
		}
		
		$rs = $DB->Execute($sql, array($projectID));
		
		$retVal = $this->LoadRecordSetEntry($rs);
		$rs->Close();
		
		return $retVal;

	}

	/**
	* Load a Project's details using an exisitng record set.
	*
	* Called by public Load() methods.
	* @param $rs The recordset running the specified query and providing data for the project.
	* @returns True on success, false otherwise (no more rows).
	*/
	
	private function LoadRecordSetEntry($rs)
	{
		if($row = $rs->FetchRow())
		{
			$this->ProjectID = $row["ID"];
			if($row["RootID"] == null) 
			{
				$this->RootID = 0;
			}
			else
			{
				$this->RootID = $row["RootID"];
			}
			$this->ParentID = $row["ParentID"];
			$this->Name = $row["Name"];
			$this->IsFolder = ($row["IsFolder"] == 1);
			$this->LocaleID = $row["LocaleID"];
			$this->LastSeen = $row["LastSeen"];
			
			if($this->Brief)
			{
				unset($this->Skeleton);
				unset($this->Document);
			}
			else
			{
				$this->Skeleton = $row["Skeleton"];
				$this->Document = $row["OrigDoc"];
			}
			return true;
		}
		else
		{
			unset($this->ProjectID);
			unset($this->RootID);
			unset($this->ParentID);
			unset($this->Name);
			unset($this->IsFolder);
			unset($this->LocaleID);
			unset($this->Skeleton);
			unset($this->Document);
			unset($this->lastSeen);
			unset($this->ProjectID);
			return false;
		}
	}

	/**
	* Removes the current project from the database.
	*
	* Throws exception if the project has child projects or contains strings.
	*/
	public function DestroyData()
	{
		$DB =& GetDbConn();
		$Tables =& GetTables();

		$children = new ProjectIterator($this->ProjectID, $this->LocaleID);
		if($children->valid())
		{
			throw new Exception("Cannot destroy data of project ".$this->ProjectID.", because it has children.");
		}

		$strings = new StringIterator($this->ProjectID);
		if($strings->valid())
		{
			throw new Exception("Cannot destroy data of project ".$this->ProjectID.", because it contains strings.");
		}
		$DB->Execute("DELETE FROM ".$Tables["Projects"]." WHERE ID = ".$this->ProjectID);

		// Clear cached projects
		ProjectIterator::ClearCache($this->LocaleID);
	}

	/* SET METHODS */

	/**
	* Set the name of this Project
	*
	* To apply changes, you must ->Save() into the database.
	* @param $name String New name for this project.
	*/
	public function SetName($name)
	{
		if(!$name)
		{
			throw new Exception("Cannot set Project Name to emtpy.");
		}
		$this->Name = $name;
	}
	
	/**
	* Set the Skeleton of this Project (usually the language file's contents,
	* with replaceable variables for translated strings) 
	*
	* To apply changes, you must ->Save() into the database.
	* @param $skeleton String Skeleton document for this project.
	*/
	public function SetSkeleton($skeleton)
	{
		$this->Skeleton = $skeleton;
	}

	/**
	* Set the original Document of this Project (usually the language file's original content).
	*
	* To apply changes, you must ->Save() into the database.
	* @param $document String Original document of the project.
	*/
	public function SetDocument($document)
	{
		$this->Document = $document;
	}


	/* GET METHODS */
	
	/**
	* Get the name of this Project
	*
	* @returns string Name of this project.
	*/
	public function GetName()
	{
		if(!isset($this->Name))
		{
			throw new Exception("Uninitialized project, cannot get Name.");
		}
		return $this->Name;
	}
	
	/**
	* Get the ID of this project. 0 if project is not yet in the database.
	*
	* @returns int ID of this project.
	*/
	public function GetID()
	{
		if(!isset($this->Name))
		{
			throw new Exception("Uninitialized project, cannot get ID.");
		}
		return $this->ProjectID;
	}

	/**
	* Get the Root project's ID containing this project.
	*
	* @returns int Root ID of this project.
	*/
	public function GetRootID()
	{
		if(!isset($this->Name))
		{
			throw new Exception("Uninitialized project, cannot get Root ID.");
		}
		return $this->RootID;
	}

	/**
	* Get the Parent ID of this project.
	*
	* @returns int Parent ID of this project.
	*/
	public function GetParentID()
	{
		if(!isset($this->Name))
		{
			throw new Exception("Uninitialized project, cannot get Parent ID.");
		}
		return $this->ParentID;
	}

	/**
	* Cheks if the project is a folder (container for projects, has no translation document).
	*
	* @returns string Returns true if this project is a folder.
	*/
	public function GetIsFolder()
	{
		if(!isset($this->Name))
		{
			throw new Exception("Uninitialized project, cannot get IsFolder.");
		}
		return $this->IsFolder;
	}
	
	/**
	* Get the locale ID of this project. 0 if project is not yet in the database.
	*
	* @returns int ID of this project.
	*/
	public function GetLocaleID()
	{
		if(!isset($this->Name))
		{
			throw new Exception("Uninitialized project, cannot get Locale ID.");
		}
		return $this->LocaleID;
	}

	/**
	* Get the skeleton of this project
	*
	* @returns string Skeleton of this project.
	*/
	public function GetSkeleton()
	{
		if(!isset($this->Name))
		{
			throw new Exception("Uninitialized project, cannot get Skeleton.");
		}
		return $this->Skeleton;
	}
	
	/**
	* Get the original document of this project
	*
	* @returns string Original document of this project.
	*/
	public function GetDocument()
	{
		if(!isset($this->Name))
		{
			throw new Exception("Uninitialized project, cannot get Document.");
		}
		return $this->Document;
	}

	/**
	* Get the last seen (last update) timestamp of this project
	*
	* Usually projects older than the root project of a tree can be considered as dead
	* projects (removed from the project, becuase they were not updated by the last import).
	* @returns timestamp The last update time of this project.
	*/
	public function GetLastSeen()
	{
		if(!isset($this->Name))
		{
			throw new Exception("Uninitialized project, cannot get LastSeen.");
		}
		return $this->LastSeen;
	}
}

?>
